import 'dart:io';
import 'dart:async';

extension WatchingRecursively on Directory {

  /// Starts to watch a directory recursively.
  /// It will recursively review directory each time, when one of `directoryEventsForReview` events will happen on any object with type of `Directory`.
  /// So, even you create subdirectory and subdirectory e.t.c - any they events will be watched.
  Stream<FileSystemEvent> watchRecursively(
      {bool followLinks = false,
      int events = FileSystemEvent.all,
      int directoryEventsForReview =
          FileSystemEvent.create | FileSystemEvent.move}) async* {
    var repeat = false;

    do {
      repeat = false;
      var stream = await _watchRecursively(
          followLinks: followLinks, events: events);

      await for (var event in stream) {
        yield event;
        if (event.isDirectory && (directoryEventsForReview & event.type) != 0) {
          repeat = true;
          break;
        }
      }
    } while (repeat);
  }

  Future<Stream<FileSystemEvent>> _watchRecursively(
      {int events = FileSystemEvent.all, bool followLinks = true}) async {
    var controller = StreamController<FileSystemEvent>();

    watch(events: events, recursive: false).listen((e) => controller.add(e));

    var dirs = list(recursive: true, followLinks: followLinks)
        .where((e) => e is Directory);

    await for (var dir in dirs) {
      dir.watch().listen((e) => controller.add(e));
    }

    return controller.stream;
  }
}
